@@ -1,10 +1,11 @@
 import os
+import re
+import shlex
 import shutil
 import stat
 import subprocess
 import tempfile
-import shlex
-from typing import Optional
+from typing import Iterable, Optional
 
 from bottles.backend.globals import (
     Paths,
@@ -23,8 +24,8 @@ from bottles.backend.utils.display import DisplayUtils
 from bottles.backend.utils.generic import detect_encoding
 from bottles.backend.utils.gpu import GPUUtils
 from bottles.backend.utils.manager import ManagerUtils
-from bottles.backend.utils.terminal import TerminalUtils
 from bottles.backend.utils.steam import SteamUtils
+from bottles.backend.utils.terminal import TerminalUtils
 
 logging = Logger()
 
@@ -37,10 +38,18 @@ class WineEnv:
     __env: dict = {}
     __result: dict = {"envs": {}, "overrides": []}
 
-    def __init__(self, clean: bool = False):
+    def __init__(self, clean: bool = False, allowed_keys: Optional[Iterable[str]] = None):
         self.__env = {}
-        if not clean:
+        if clean:
+            return
+
+        if allowed_keys is None:
             self.__env = os.environ.copy()
+            return
+
+        for key in allowed_keys:
+            if key in os.environ:
+                self.__env[key] = os.environ[key]
 
     def add(self, key, value, override=False):
         if key in self.__env:
@@ -81,6 +90,42 @@ class WineEnv:
         return key in self.__env
 
 
+def apply_wayland_preferences(env: "WineEnv", params) -> None:
+    if not getattr(params, "wayland", False):
+        return
+    if DisplayUtils.display_server_type() != "wayland":
+        return
+    wayland_display = os.environ.get("WAYLAND_DISPLAY")
+    if not env.has("WAYLAND_DISPLAY") and wayland_display:
+        env.add("WAYLAND_DISPLAY", wayland_display, override=True)
+    if env.has("WAYLAND_DISPLAY") or wayland_display:
+        env.remove("DISPLAY")
+
+
+def _needs_steam_virtual_gamepad_workaround(runner_name: Optional[str]) -> bool:
+    """Return True if the runner should force SteamVirtualGamepadInfo."""
+
+    if not runner_name:
+        return False
+
+    normalized = runner_name.lower()
+    if not any(
+        prefix in normalized for prefix in ("ge-proton", "proton-ge", "wine-ge", "soda")
+    ):
+        return False
+
+    match = re.search(r"(\d+)", normalized)
+    if not match:
+        return False
+
+    try:
+        major = int(match.group(1))
+    except ValueError:
+        return False
+
+    return major <= 8
+
+
 class WineCommand:
     """
     This class is used to run a wine command with a custom environment.
@@ -99,6 +144,8 @@ class WineCommand:
         minimal: bool = False,  # avoid gamemode/gamescope usage
         pre_script: Optional[str] = None,
         post_script: Optional[str] = None,
+        pre_script_args: Optional[str] = None,
+        post_script_args: Optional[str] = None,
         cwd: Optional[str] = None,
     ):
         _environment = environment.copy()
@@ -113,7 +160,12 @@ class WineCommand:
             else self.config.Parameters.gamescope
         )
         self.command = self.get_cmd(
-            command, pre_script, post_script, environment=_environment
+            command,
+            pre_script,
+            post_script,
+            pre_script_args,
+            post_script_args,
+            environment=_environment,
         )
         self.terminal = terminal
         self.env = self.get_env(_environment)
@@ -160,8 +212,13 @@ class WineCommand:
         return_steam_env: bool = False,
         return_clean_env: bool = False,
     ) -> dict:
-        env = WineEnv(clean=return_steam_env or return_clean_env)
         config = self.config
+        clean_env = return_steam_env or return_clean_env
+        allowed_env_keys: Optional[Iterable[str]] = None
+        if not clean_env and getattr(config, "Limit_System_Environment", False):
+            allowed_env_keys = config.Inherited_Environment_Variables
+
+        env = WineEnv(clean=clean_env, allowed_keys=allowed_env_keys)
         arch = config.Arch
         params = config.Parameters
 
@@ -194,6 +251,11 @@ class WineCommand:
         ld = []
 
         # Bottle environment variables
+        if _needs_steam_virtual_gamepad_workaround(config.Runner) and not env.has(
+            "SteamVirtualGamepadInfo"
+        ):
+            env.add("SteamVirtualGamepadInfo", "", override=True)
+
         if config.Environment_Variables:
             for key, value in config.Environment_Variables.items():
                 env.add(key, value, override=True)
@@ -280,19 +342,20 @@ class WineCommand:
             ]
             gst_libs = ["lib/gstreamer-1.0", "lib32/gstreamer-1.0"]
 
-        for lib in runner_libs:
-            _path = os.path.join(runner_path, lib)
-            if os.path.exists(_path):
-                ld.append(_path)
+        if not config.Runner.startswith("sys-"):
+            for lib in runner_libs:
+                _path = os.path.join(runner_path, lib)
+                if os.path.exists(_path):
+                    ld.append(_path)
 
-        # Embedded GStreamer environment variables
-        if not env.has("BOTTLES_USE_SYSTEM_GSTREAMER") and not return_steam_env:
-            gst_env_path = []
-            for lib in gst_libs:
-                if os.path.exists(os.path.join(runner_path, lib)):
-                    gst_env_path.append(os.path.join(runner_path, lib))
-            if len(gst_env_path) > 0:
-                env.add("GST_PLUGIN_SYSTEM_PATH", ":".join(gst_env_path), override=True)
+            # Embedded GStreamer environment variables
+            if not env.has("BOTTLES_USE_SYSTEM_GSTREAMER") and not return_steam_env:
+                gst_env_path = []
+                for lib in gst_libs:
+                    if os.path.exists(os.path.join(runner_path, lib)):
+                        gst_env_path.append(os.path.join(runner_path, lib))
+                if len(gst_env_path) > 0:
+                    env.add("GST_PLUGIN_SYSTEM_PATH", ":".join(gst_env_path), override=True)
 
         # DXVK environment variables
         if params.dxvk and not return_steam_env:
@@ -444,6 +507,8 @@ class WineCommand:
             # Wine arch
             env.add("WINEARCH", arch)
 
+        apply_wayland_preferences(env, params)
+
         return env.get()["envs"]
 
     def _get_runner_info(self) -> tuple[str, str]:
@@ -489,6 +554,8 @@ class WineCommand:
         command,
         pre_script: Optional[str] = None,
         post_script: Optional[str] = None,
+        pre_script_args: Optional[str] = None,
+        post_script_args: Optional[str] = None,
         return_steam_cmd: bool = False,
         return_clean_cmd: bool = False,
         environment: Optional[dict] = None,
@@ -598,10 +665,18 @@ class WineCommand:
                 environment.update(extracted_env)
 
         if post_script not in (None, ""):
-            command = f"{command} ; sh '{post_script}'"
+            post_cmd_parts = [post_script]
+            if post_script_args not in (None, ""):
+                post_cmd_parts.extend(shlex.split(post_script_args))
+            post_cmd = " ".join(shlex.quote(part) for part in post_cmd_parts)
+            command = f"{command} ; sh {post_cmd}"
 
         if pre_script not in (None, ""):
-            command = f"sh '{pre_script}' ; {command}"
+            pre_cmd_parts = [pre_script]
+            if pre_script_args not in (None, ""):
+                pre_cmd_parts.extend(shlex.split(pre_script_args))
+            pre_cmd = " ".join(shlex.quote(part) for part in pre_cmd_parts)
+            command = f"sh {pre_cmd} ; {command}"
 
         return command
 
@@ -614,6 +689,8 @@ class WineCommand:
             gamescope_cmd = [gamescope_available]
             if return_steam_cmd:
                 gamescope_cmd = ["gamescope"]
+            if params.gamescope_custom_options:
+                gamescope_cmd.append(params.gamescope_custom_options)
             if params.gamescope_fullscreen:
                 gamescope_cmd.append("-f")
             if params.gamescope_borderless:
@@ -681,7 +758,7 @@ class WineCommand:
             envs=self.env,
             chdir=self.cwd,
             share_paths_rw=[ManagerUtils.get_bottle_path(self.config)],
-            share_paths_ro=[Paths.runners, Paths.temp],
+            share_paths_ro=[p for p in [Paths.runners, Paths.temp] if p],
             share_net=self.config.Sandbox.share_net,
             share_sound=self.config.Sandbox.share_sound,
         )
@@ -698,6 +775,9 @@ class WineCommand:
                 False, message="runner or env is not ready, Wine command terminated."
             )
 
+        # Log the final command that will be executed
+        logging.info(f"Executing command: {self.command}")
+
         if vmtouch_available and self.config.Parameters.vmtouch and not self.terminal:
             self._vmtouch_preload()
 
